@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-gestures-extension.c,v 1.3 2003/11/20 23:16:58 noda Exp $
+ *  $Id: ephy-gestures-extension.c,v 1.8 2004/02/24 17:47:18 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -25,6 +25,8 @@
 
 #include "ephy-gestures-extension.h"
 #include "ephy-gesture.h"
+
+#include "ephy-file-helpers.h"
 #include "ephy-debug.h"
 
 #include <epiphany/ephy-embed.h>
@@ -53,7 +55,7 @@ struct EphyGesturesExtensionPrivate
 static GObjectClass *parent_class = NULL;
 
 static void ephy_gestures_extension_class_init	(EphyGesturesExtensionClass *klass);
-static void ephy_gestures_extension_iface_init	(EphyExtensionClass *iface);
+static void ephy_gestures_extension_iface_init	(EphyExtensionIface *iface);
 static void ephy_gestures_extension_init	(EphyGesturesExtension *extension);
 
 static GType type = 0;
@@ -150,18 +152,15 @@ load_one_gesture (EphyGesturesExtension *extension,
 }
 
 static void
-load_gestures (EphyGesturesExtension *extension)
+load_gestures (EphyGesturesExtension *extension,
+	       const char *filename)
 {
-	char *filename;
 	xmlDocPtr doc;
 	xmlNodePtr root, child;
 	xmlChar *tmp;
 
-	filename = g_build_filename (SHARE_DIR, EPHY_GESTURES_XML_FILE, NULL);
-
 	if (g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE)
 	{
-		g_free (filename);
 		return;
 	}
 
@@ -170,14 +169,14 @@ load_gestures (EphyGesturesExtension *extension)
 	{
 		g_warning ("Failed to load the gestures definitions from %s\n",
 			   filename);
-		g_free (filename);
 		return;
 	}
 
 	root = xmlDocGetRootElement (doc);
 	if (root == NULL || strcmp (root->name, EPHY_GESTURES_XML_ROOT) != 0)
 	{
-		g_warning ("Gestures definitions file %s has wrong format %s\n",
+		g_warning ("Gestures definitions file %s has wrong format '%s'"
+		           "(expected " EPHY_GESTURES_XML_ROOT ")\n",
 			   filename, root ? (char*) root->name : "(unknown)");
 		goto out;
 	}
@@ -185,7 +184,8 @@ load_gestures (EphyGesturesExtension *extension)
 	tmp = xmlGetProp (root, "version");
 	if (tmp  == NULL || strcmp (tmp, EPHY_GESTURES_XML_VERSION) != 0)
 	{
-		g_warning ("Gestures definitions file %s has wrong format version %s\n",
+		g_warning ("Gestures definitions file %s has wrong format version %s"
+			   "(expected " EPHY_GESTURES_XML_VERSION ")\n",
 			   filename, tmp ? (char*) tmp : "(unknown)");
 		goto out;
 	}
@@ -202,24 +202,32 @@ out:
 	xmlFreeDoc (doc);
 
 	xmlFree (tmp);
-	g_free (filename);
 }
 
 static void
 ephy_gestures_extension_init (EphyGesturesExtension *extension)
 {
+	char *filename;
+
 	extension->priv = EPHY_GESTURES_EXTENSION_GET_PRIVATE (extension);
 
 	extension->priv->gestures =
 		g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	load_gestures (extension);
+	/* load the system gestures definitions */
+	filename = g_build_filename (SHARE_DIR, EPHY_GESTURES_XML_FILE, NULL);
+	load_gestures (extension, filename);
+	g_free (filename);
+
+	/* now load the user's gestures definitions */
+	filename = g_build_filename (ephy_dot_dir (), EPHY_GESTURES_XML_FILE, NULL);
+	load_gestures (extension, filename);
+	g_free (filename);
 }
 
 static void
 ephy_gestures_extension_finalize (GObject *object)
 {
-
 	EphyGesturesExtension *extension = EPHY_GESTURES_EXTENSION (object);
 
 	g_hash_table_destroy (extension->priv->gestures);
@@ -312,7 +320,7 @@ dom_mouse_down_cb (EphyEmbed *embed,
 		EphyTab *tab;
 		guint x, y;
 
-		tab = EPHY_TAB (g_object_get_data (G_OBJECT (embed), "EphyTab"));
+		tab = ephy_tab_for_embed (embed);
 		g_return_if_fail (EPHY_IS_TAB (tab));
 
 		window = ephy_tab_get_window (tab);
@@ -334,28 +342,34 @@ dom_mouse_down_cb (EphyEmbed *embed,
 
 static void
 tab_added_cb (GtkWidget *notebook,
-	      GtkWidget *child,
+	      EphyTab *tab,
 	      EphyGesturesExtension *extension)
 {
-	EphyTab *tab;
+	EphyEmbed *embed;
 
-	tab = EPHY_TAB (g_object_get_data (G_OBJECT (child), "EphyTab"));
+	g_return_if_fail (EPHY_IS_TAB (tab));
 
-	g_signal_connect (G_OBJECT (child), "ge_dom_mouse_down",
+	embed = ephy_tab_get_embed (tab);
+	g_return_if_fail (EPHY_IS_EMBED (embed));
+
+	g_signal_connect (embed, "ge_dom_mouse_down",
 			  G_CALLBACK (dom_mouse_down_cb), extension);
 }
 
 static void
 tab_removed_cb (GtkWidget *notebook,
-		GtkWidget *child,
+		EphyTab *tab,
 		EphyGesturesExtension *extension)
 {
-	EphyTab *tab;
+	EphyEmbed *embed;
 
-	tab = EPHY_TAB (g_object_get_data (G_OBJECT (child), "EphyTab"));
+	g_return_if_fail (EPHY_IS_TAB (tab));
+
+	embed = ephy_tab_get_embed (tab);
+	g_return_if_fail (EPHY_IS_EMBED (embed));
 
 	g_signal_handlers_disconnect_by_func
-		(child, G_CALLBACK (dom_mouse_down_cb), extension);
+		(embed, G_CALLBACK (dom_mouse_down_cb), extension);
 }
 
 static void
@@ -387,7 +401,7 @@ impl_detach_window (EphyExtension *extension,
 }
 
 static void
-ephy_gestures_extension_iface_init (EphyExtensionClass *iface)
+ephy_gestures_extension_iface_init (EphyExtensionIface *iface)
 {
 	iface->attach_window = impl_attach_window;
 	iface->detach_window = impl_detach_window;
