@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-popup-blocker-extension.c,v 1.7 2004/03/06 00:15:18 adamh Exp $
+ *  $Id: ephy-popup-blocker-extension.c,v 1.19.2.1 2004/03/22 12:54:01 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,7 +27,7 @@
 #include "mozilla-helpers.h"
 #include "ephy-popup-blocker-extension.h"
 #include "ephy-popup-blocker-icon.h"
-#include "ephy-popup-blocker-popup.h"
+#include "ephy-popup-blocker-list.h"
 #include "ephy-file-helpers.h"
 #include "ephy-debug.h"
 
@@ -60,9 +60,9 @@ static void clear_popup_permissions (void);
 
 static GtkToggleActionEntry action_entries [] =
 {
-	{ "PopupBlocker", NULL, N_("Blo_ck Popups from this Site"),
+	{ "PopupBlocker", NULL, N_("Popup _Windows"),
 	  NULL, /* shortcut key */
-	  N_("Block or unblock popup windows"),
+	  N_("Show or hide unrequested popup windows from this site"),
 	  NULL, TRUE }
 };
 static const guint n_action_entries = G_N_ELEMENTS (action_entries);
@@ -116,6 +116,31 @@ ephy_popup_blocker_extension_register_type (GTypeModule *module)
 	return type;
 }
 
+EphyPopupBlockerIcon *
+get_icon_for_embed (EphyEmbed *embed)
+{
+	EphyWindow *window;
+	EphyTab *tab;
+	GtkWidget *statusbar;
+	gpointer icon;
+
+	g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
+
+	tab = ephy_tab_for_embed (embed);
+	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
+
+	window = ephy_tab_get_window (tab);
+	g_return_val_if_fail (EPHY_IS_WINDOW (window), NULL);
+
+	statusbar = ephy_window_get_statusbar (window);
+	g_return_val_if_fail (EPHY_IS_STATUSBAR (statusbar), NULL);
+
+	icon = g_object_get_data (G_OBJECT (statusbar), "popup-blocker-icon");
+	g_return_val_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon), NULL);
+
+	return (EphyPopupBlockerIcon *) icon;
+}
+
 static void
 update_action (EphyWindow *window, const char *address)
 {
@@ -152,8 +177,7 @@ update_action (EphyWindow *window, const char *address)
 					 G_CALLBACK (action_activate_cb),
 					 window);
 
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
-				      allow == FALSE);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), allow);
 
 	g_signal_handlers_unblock_by_func (G_OBJECT (action),
 					   G_CALLBACK (action_activate_cb),
@@ -172,31 +196,21 @@ update_action (EphyWindow *window, const char *address)
 static void
 update_action_without_address (EphyWindow *window)
 {
-	/*
-	 * FIXME: The address used to update_action could be inaccurate.
-	 * We can't use ephy_tab_get_embed() because of bug #119047
-	 */
+	EphyEmbed *embed;
+	char *address;
 
-	EphyTab *tab;
-	const char *address;
+	embed = ephy_window_get_active_embed (window);
 
-	tab = ephy_window_get_active_tab (window);
+	if (embed == NULL) return; /* Happens on startup */
 
-	if (tab == NULL) return; /* Happens on startup */
+	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	g_return_if_fail (EPHY_IS_TAB (tab));
-
-	address = ephy_tab_get_location (tab);
+	address = mozilla_get_location (embed);
 	g_return_if_fail (address != NULL);
 
 	update_action (window, address);
-}
 
-static void
-update_all_actions_foreach_func (gpointer data,
-				 gpointer user_data)
-{
-	update_action_without_address (EPHY_WINDOW (data));
+	g_free (address);
 }
 
 static void
@@ -214,7 +228,7 @@ update_all_actions (GConfClient *client,
 	session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
 	windows = ephy_session_get_windows (session);
 
-	g_list_foreach (windows, (GFunc) update_all_actions_foreach_func, NULL);
+	g_list_foreach (windows, (GFunc) update_action_without_address, NULL);
 }
 
 static void
@@ -294,49 +308,14 @@ clear_popup_permissions (void)
 	g_list_free (list);
 }
 
-EphyWindow *
-get_window_for_embed (EphyEmbed *embed)
-{
-	EphyTab *tab;
-	EphyWindow *window;
-
-	g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
-
-	tab = ephy_tab_for_embed (embed);
-	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
-
-	window = ephy_tab_get_window (tab);
-	g_return_val_if_fail (EPHY_IS_WINDOW (window), NULL);
-
-	return window;
-}
-
-EphyPopupBlockerIcon *
-get_icon_for_embed (EphyEmbed *embed)
-{
-	EphyWindow *window;
-	gpointer icon;
-
-	g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
-
-	window = get_window_for_embed (embed);
-	g_return_val_if_fail (EPHY_IS_WINDOW (window), NULL);
-
-	icon = g_object_get_data (G_OBJECT (window), "popup-blocker-icon");
-	g_return_val_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon), NULL);
-
-	return (EphyPopupBlockerIcon *) icon;
-}
-
 static void
 location_cb (EphyEmbed *embed,
 	     const char *address,
 	     EphyTab *tab)
 {
 	EphyWindow *window;
-	EphyPopupBlockerIcon *icon;
 	GtkWidget *statusbar;
-	GSList *prev_blocked_popups;
+	EphyPopupBlockerList *popups;
 
 	if (tab == NULL || embed == NULL) return;
 	g_return_if_fail (EPHY_IS_TAB (tab));
@@ -347,17 +326,10 @@ location_cb (EphyEmbed *embed,
 
 	update_action (window, address);
 
-	icon = get_icon_for_embed (embed);
-	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon));
+	popups = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
+	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_LIST (popups));
 
-	ephy_popup_blocker_icon_set_popups (icon, NULL);
-
-	prev_blocked_popups = g_object_get_data (G_OBJECT (embed),
-						 "popup-blocker-popups");
-	g_slist_foreach (prev_blocked_popups, (GFunc) g_object_unref, NULL);
-	g_slist_free (prev_blocked_popups);
-
-	g_object_set_data (G_OBJECT (embed), "popup-blocker-popups", NULL);
+	ephy_popup_blocker_list_reset (popups);
 }
 
 void
@@ -366,24 +338,14 @@ ephy_popup_blocker_extension_block (EphyEmbed *embed,
 				    const char *features)
 {
 	EphyWindow *window;
-	EphyPopupBlockerIcon *icon;
-	EphyPopupBlockerPopup *new_popup;
-	GSList *popups;
+	EphyPopupBlockerList *popups;
 
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	new_popup = ephy_popup_blocker_popup_new (uri, features);
+	popups = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
+	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_LIST (popups));
 
-	popups = g_object_get_data (G_OBJECT (embed), "popup-blocker-popups");
-
-	popups = g_slist_prepend (popups, new_popup);
-
-	g_object_set_data (G_OBJECT (embed), "popup-blocker-popups", popups);
-
-	icon = get_icon_for_embed (embed);
-	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon));
-
-	ephy_popup_blocker_icon_set_popups (icon, popups);
+	ephy_popup_blocker_list_insert (popups, uri, features);
 }
 
 static void
@@ -391,25 +353,29 @@ action_activate_cb (GtkAction *action,
 		    EphyWindow *window)
 {
 	const char *address;
-	EphyTab *tab;
+	EphyEmbed *embed;
+	EphyPopupBlockerList *list;
 	EphyPermissionManager *permission_manager;
 	EphyPermission allow;
 
-	tab = ephy_window_get_active_tab (window);
-	g_return_if_fail (EPHY_IS_TAB (tab));
+	embed = ephy_window_get_active_embed(window);
+	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	address = ephy_tab_get_location (tab);
+	address = ephy_embed_get_location (embed, TRUE);
 	g_return_if_fail (address != NULL);
 
+	list = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
+	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_LIST (list));
+
 	permission_manager = EPHY_PERMISSION_MANAGER (ephy_embed_shell_get_embed_single (EPHY_EMBED_SHELL (ephy_shell)));
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)) == TRUE)
 	{
-		allow = EPHY_PERMISSION_DENIED;
+		allow = EPHY_PERMISSION_ALLOWED;
 	}
 	else
 	{
-		allow = EPHY_PERMISSION_ALLOWED;
+		allow = EPHY_PERMISSION_DENIED;
 	}
 
 	LOG ("from now on, address '%s' will %s popups", address,
@@ -417,19 +383,32 @@ action_activate_cb (GtkAction *action,
 
 	ephy_permission_manager_add (permission_manager, address,
 				     EPT_POPUP, allow); 
+
+	if (allow == EPHY_PERMISSION_ALLOWED)
+	{
+		ephy_popup_blocker_list_show_all (list);
+	}
+	else
+	{
+		ephy_popup_blocker_list_hide_all (list);
+	}
 }
 
 static void
 create_statusbar_icon (EphyWindow *window)
 {
 	EphyPopupBlockerIcon *icon;
+	GtkWidget *statusbar;
 
 	g_return_if_fail (EPHY_IS_WINDOW (window));
 
-	icon = ephy_popup_blocker_icon_new (window);
+	statusbar = ephy_window_get_statusbar (window);
+	g_return_if_fail (EPHY_IS_STATUSBAR (statusbar));
+
+	icon = ephy_popup_blocker_icon_new (statusbar);
 	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon));
 
-	g_object_set_data (G_OBJECT (window), "popup-blocker-icon", icon);
+	g_object_set_data (G_OBJECT (statusbar), "popup-blocker-icon", icon);
 }
 
 static void
@@ -447,10 +426,26 @@ register_mozilla (EphyEmbed *embed)
 {
 	PopupListenerFreeData *data;
 
+	if (GTK_WIDGET_REALIZED (embed) == FALSE)
+	{
+		g_signal_connect (embed, "realize",
+				  G_CALLBACK (register_mozilla), NULL);
+		return;
+	}
+	else
+	{
+		g_signal_handlers_disconnect_by_func
+			(embed, G_CALLBACK (register_mozilla), NULL);
+	}
+
 	LOG ("register_mozilla with EphyEmbed at %x", embed)
 
 	g_return_if_fail (EPHY_IS_EMBED (embed));
-	g_return_if_fail (GTK_WIDGET_REALIZED (embed));
+
+	if (g_object_get_data (G_OBJECT (embed), "popup-blocker-listener-data"))
+	{
+		return;
+	}
 
 	data = mozilla_register_popup_listener (embed);
 	g_return_if_fail (data != NULL);
@@ -461,83 +456,122 @@ register_mozilla (EphyEmbed *embed)
 				data, (GDestroyNotify) unregister_mozilla);
 }
 
+static void
+new_window_cb (EphyEmbed *embed,
+	       EphyEmbed **new_embed,
+	       guint chromemask,
+	       EphyPopupBlockerList *popups)
+{
+	EphyWindow *window;
+	EphyTab *tab;
+
+	g_return_if_fail (EPHY_IS_EMBED (embed));
+	g_return_if_fail (new_embed != NULL);
+	g_return_if_fail (EPHY_IS_EMBED (*new_embed));
+	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_LIST (popups));
+
+	tab = ephy_tab_for_embed (*new_embed);
+	g_return_if_fail (EPHY_IS_TAB (tab));
+
+	window = ephy_tab_get_window (tab);
+	g_return_if_fail (EPHY_IS_WINDOW (window));
+
+	ephy_popup_blocker_list_insert_window (popups, window);
+}
+
 static void
 tab_added_cb (GtkWidget *notebook,
 	      EphyTab *tab,
-	      EphyPopupBlockerExtension *extension)
+	      EphyWindow *window)
 {
 	EphyEmbed *embed;
+	EphyPopupBlockerList *popups;
+	EphyPopupBlockerIcon *icon;
 
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
+	LOG ("tab_added_cb: tab %p, window %p\n", tab, window)
+
 	embed = ephy_tab_get_embed (tab);
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	g_signal_connect (embed, "realize", G_CALLBACK (register_mozilla),
-			  NULL);
+	popups = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
+
+	if (popups == NULL)
+	{
+		popups = ephy_popup_blocker_list_new (embed);
+		g_return_if_fail (EPHY_IS_POPUP_BLOCKER_LIST (popups));
+
+		g_object_set_data_full (G_OBJECT (embed),
+					"popup-blocker-list", popups,
+					(GDestroyNotify) g_object_unref);
+	}
+
+	icon = get_icon_for_embed (embed);
+	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon));
+
+	ephy_popup_blocker_icon_set_popups (icon, popups);
+
+	register_mozilla (embed);
 
 	g_signal_connect (embed, "ge_location",
 			  G_CALLBACK (location_cb), tab);
+	g_signal_connect_object (embed, "ge_new_window",
+				 G_CALLBACK (new_window_cb), popups,
+				 G_CONNECT_AFTER);
 }
 
 static void
 tab_removed_cb (GtkWidget *notebook,
 		EphyTab *tab,
-		EphyPopupBlockerExtension *extension)
+		EphyWindow *window)
 {
 	EphyEmbed *embed;
-	EphyPopupBlockerIcon *icon;
-	GSList *blocked_popups;
+	EphyPopupBlockerList *popups;
 
 	g_return_if_fail (EPHY_IS_TAB (tab));
 
+	LOG ("tab_removed_cb: tab %p, window %p\n", tab, window)
+
 	embed = ephy_tab_get_embed (tab);
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	icon = get_icon_for_embed (embed);
-	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon));
-
-	ephy_popup_blocker_icon_set_popups (icon, NULL);
-
-	blocked_popups = g_object_get_data (G_OBJECT (embed),
-					    "popup-blocker-popups");
-	g_slist_foreach (blocked_popups, (GFunc) g_object_unref, NULL);
-	g_slist_free (blocked_popups);
+	popups = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
+	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_LIST (popups));
 
 	g_signal_handlers_disconnect_by_func
 		(embed, G_CALLBACK (location_cb), tab);
+	g_signal_handlers_disconnect_by_func
+		(embed, G_CALLBACK (new_window_cb), popups);
 }
 
 static void
 switch_page_cb (GtkNotebook *notebook,
 		GtkNotebookPage *page,
 		guint page_num,
-		EphyPopupBlockerExtension *extension)
+		EphyWindow *window)
 {
-	GtkWidget *toplevel;
 	EphyEmbed *embed;
 	EphyPopupBlockerIcon *icon;
-	GSList *blocked_popups;
+	EphyPopupBlockerList *popups;
 
 	LOG ("switch_page_cb: page %x", page)
 
-	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (notebook));
-	g_return_if_fail (EPHY_IS_WINDOW (toplevel));
-	if (GTK_WIDGET_REALIZED (toplevel) == FALSE) return; /* on startup */
+	g_return_if_fail (EPHY_IS_WINDOW (window));
 
-	embed = ephy_window_get_active_embed (EPHY_WINDOW (toplevel));
+	if (GTK_WIDGET_REALIZED (window) == FALSE) return; /* on startup */
+
+	embed = ephy_window_get_active_embed (window);
 	g_return_if_fail (EPHY_IS_EMBED (embed));
 
-	icon = EPHY_POPUP_BLOCKER_ICON (g_object_get_data
-			(G_OBJECT (toplevel), "popup-blocker-icon"));
+	icon = get_icon_for_embed (embed);
 	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon));
 
-	blocked_popups = g_object_get_data (G_OBJECT (embed),
-					    "popup-blocker-popups");
+	popups = g_object_get_data (G_OBJECT (embed), "popup-blocker-list");
 
-	ephy_popup_blocker_icon_set_popups (icon, blocked_popups);
+	ephy_popup_blocker_icon_set_popups (icon, popups);
 
-	update_action_without_address (EPHY_WINDOW (toplevel));
+	update_action_without_address (window);
 }
 
 static void
@@ -551,8 +585,6 @@ impl_attach_window (EphyExtension *extension,
 	GtkUIManager *manager;
 	guint merge_id;
 
-	g_object_ref (window);
-
 	create_statusbar_icon (window);
 
 	/* Merge the UI */
@@ -571,10 +603,10 @@ impl_attach_window (EphyExtension *extension,
 
 	gtk_ui_manager_add_ui (manager, merge_id, "/menubar/ViewMenu",
 			       "PopupBlockerSep", NULL,
-			       GTK_UI_MANAGER_SEPARATOR, TRUE);
+			       GTK_UI_MANAGER_SEPARATOR, FALSE);
 	gtk_ui_manager_add_ui (manager, merge_id, "/menubar/ViewMenu",
 			       "PopupBlocker", "PopupBlocker",
-			       GTK_UI_MANAGER_MENUITEM, TRUE);
+			       GTK_UI_MANAGER_MENUITEM, FALSE);
 
 	/*
 	 * Connect the GtkToggleAction's activate signal
@@ -593,18 +625,17 @@ impl_attach_window (EphyExtension *extension,
 	notebook = ephy_window_get_notebook (window);
 
 	g_signal_connect_after (notebook, "tab_added",
-				G_CALLBACK (tab_added_cb), extension);
+				G_CALLBACK (tab_added_cb), window);
 	g_signal_connect_after (notebook, "tab_removed",
-				G_CALLBACK (tab_removed_cb), extension);
+				G_CALLBACK (tab_removed_cb), window);
 	g_signal_connect_after (notebook, "switch_page",
-				G_CALLBACK (switch_page_cb), extension);
+				G_CALLBACK (switch_page_cb), window);
 }
 
 static void
 impl_detach_window (EphyExtension *extension,
 		    EphyWindow *window)
 {
-	EphyPopupBlockerIcon *icon;
 	GtkWidget *notebook;
 
 	g_return_if_fail (EPHY_IS_WINDOW (window));
@@ -612,18 +643,11 @@ impl_detach_window (EphyExtension *extension,
 	notebook = ephy_window_get_notebook (window);
 
 	g_signal_handlers_disconnect_by_func
-		(notebook, G_CALLBACK (tab_added_cb), extension);
+		(notebook, G_CALLBACK (tab_added_cb), window);
 	g_signal_handlers_disconnect_by_func
-		(notebook, G_CALLBACK (tab_removed_cb), extension);
+		(notebook, G_CALLBACK (tab_removed_cb), window);
 	g_signal_handlers_disconnect_by_func
-		(notebook, G_CALLBACK (switch_page_cb), extension);
-
-	icon = g_object_get_data (G_OBJECT (window), "popup-blocker-icon");
-	g_return_if_fail (EPHY_IS_POPUP_BLOCKER_ICON (icon));
-
-	g_object_unref (icon);
-
-	g_object_unref (window);
+		(notebook, G_CALLBACK (switch_page_cb), window);
 }
 
 static void
